#ifndef CAMERAFOCUS_H
#define CAMERAFOCUS_H

#include <QDialog>
#include <QTimer>
#include <QThread>
#include <QAbstractButton>
#include <sandboxSetup.h>
#include "monitorgui.h"
#include "qtfullscreen.h"
#include "subapp.h"

namespace Ui {
class CameraFocus;
}


class CameraFocus : public SubApp
{
    Q_OBJECT

public:
    explicit CameraFocus(SandboxSetup *sandbox, MonitorGui *_mg, QWidget *parent = 0);
    ~CameraFocus();
    void refreshFrame();
    void valideRoutine();
    void cancelRoutine();


private slots:
    void on_sbxBrightness_valueChanged(int arg1);

    void on_btnReset_clicked();

    void on_sbxCannyThreshold_valueChanged(int arg1);

    void on_sbxAccThreshold_valueChanged(int arg1);

    void on_sldContrast_sliderMoved(int position);

    void on_sbxContrast_valueChanged(double arg1);

    void on_sldBrightness_sliderMoved(int position);

    void on_sldMinDistance_sliderMoved(int position);

    void on_sbxMinDistance_valueChanged(int arg1);

    void on_sldCannyThreshold_sliderMoved(int position);

    void on_sldAccThreshold_sliderMoved(int position);

    void on_sldMinRadius_sliderMoved(int position);

    void on_sbxMinRadius_valueChanged(int arg1);

    void on_sldMaxRadius_sliderMoved(int position);

    void on_sbxMaxRadius_valueChanged(int arg1);

    void on_ckbxTargetSize_clicked(bool checked);

    void on_ckbxAdvanced_clicked(bool checked);

protected:
    void showEvent(QShowEvent *event);
    void closeEvent(QCloseEvent *event);

private:

    class RefreshFrame : public QThread
    {
        public:
            explicit RefreshFrame(CameraFocus *camera);
            void run();

        private:
            CameraFocus *camera;
    };

    Ui::CameraFocus *ui;
    SandboxSetup *setup;
    MonitorGui *mg;
    RefreshFrame *myThread;
    QTimer *frameTimer;
    QtFullScreen *blackScreen;
    FrameProcessProfil defaultProfil;

    void startCapture();
    void loadProfil(FrameProcessProfil *profilLoaded, FrameProcessProfil *profilSaved);
    void initCameraParams();
    void switchMode(bool isRestricted);
};


#endif // CAMERAFOCUS_H
