#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->apps->setGeometry(QRect(0, 0, 670, 630));
    ui->btnPrev->setVisible(false);
    ui->btnNext->setVisible(false);
    ui->btnFinish->setVisible(false);

    setup = new SandboxSetup;
    init = new InitCamera(setup);
    mg = new MonitorGui(setup);
    pg = new ProjectionGui(setup, mg);
    cf = new CameraFocus(setup, mg);
    cm = new CroppingMask(setup, mg);
    bl = new BeamerLocationGui(setup, mg);
    scfg = new SaveConfigGui(setup);

    applist.push_back(init);
    applist.push_back(mg);
    applist.push_back(pg);
    applist.push_back(cf);
    applist.push_back(cm);
    applist.push_back(bl);
    applist.push_back(scfg);

    for(SubApp* app : applist){
        connect(app, &SubApp::sendNotif, this, &MainWindow::receiveNotif);
    }
    this->setFocusPolicy(Qt::StrongFocus);
}

MainWindow::~MainWindow()
{
    delete scfg;
    delete bl;
    delete cm;
    delete cf;
    delete pg;
    delete mg;
    delete init;
    delete setup;
    delete ui;
}

void MainWindow::showEvent(QShowEvent *event){
    QWidget::showEvent(event);
    loadApp();
}

void MainWindow::closeEvent(QCloseEvent *event){
    closeApp();
    if(!verifyApp())
        exit_msg("Failed setup at step " + std::to_string(step) + " : " + applist.at(step)->getErrorMessage());

    QWidget::closeEvent(event);
}

void MainWindow::receiveNotif(int state){

    // active app : InitCamera
    if( strcmp( applist.at(step)->metaObject()->className(), init->metaObject()->className() ) == 0 ){
        if(state){
            ui->btnFinish->setVisible(true);
        }else{
            ui->btnNext->setVisible(true);
            nextApp();
        }
    }
    // active app : BeamerLocation
    else if( strcmp( applist.at(step)->metaObject()->className(), bl->metaObject()->className() ) == 0 ){
            ui->btnNext->setVisible(state);
    }
    // active app : SaveConfig
    else if( strcmp( applist.at(step)->metaObject()->className(), scfg->metaObject()->className() ) == 0 ){
        ui->btnFinish->setVisible(true);
    }
}

void MainWindow::exit_msg(std::string msg){
    setup->getCamera()->stop();
    std::cout << msg << std::endl;
    exit(1);
}


//
//  STEPS
//

void MainWindow::closeApp(){
    applist.at(step)->close();
    if(ui->apps->count() > 0){
        ui->apps->removeWidget(applist.at(step));
    }
}

void MainWindow::loadApp(){
    ui->apps->addWidget(applist.at(step));
    ui->lblTitle->setText( QString( (std::to_string(step+1) + ". " + applist.at(step)->getTitle()).c_str() ) );
    applist.at(step)->show();
}

bool MainWindow::verifyApp(){
    return applist.at(step)->checkRoutine();
}

void MainWindow::nextApp(){
    if(verifyApp()){
        applist.at(step)->valideRoutine();
        closeApp();
        if((uint)step < applist.size()-1){
            step++;
            loadApp();
        }
    }
}

void MainWindow::prevApp(){
    if(step > 0){
        applist.at(step)->cancelRoutine();
        closeApp();
        step--;
        loadApp();
    }
}


void MainWindow::on_btnNext_clicked()
{
    nextApp();
    displayButtons();
}

void MainWindow::on_btnPrev_clicked()
{
    prevApp();
    displayButtons();
}

void MainWindow::displayButtons(){

    // button previous
    ui->btnPrev->setVisible( (1 < step && step < 6) );

    // button next
    ui->btnNext->setVisible( (0 < step && step < 5) );
}

void MainWindow::on_btnFinish_clicked()
{
    this->close();
}
