#include "maskedit.h"
#include "ui_maskedit.h"

MaskEdit::MaskEdit(const QRect geo, QWidget *parent) :
    QFrame(parent),
    ui(new Ui::MaskEdit)
{
    this->installEventFilter(this);
    ui->setupUi(this);
    // set size like the parent widget's size
    setGeometry(0,0, geo.width(), geo.height());
    ui->lblFrame->setGeometry(0,0, width(), height());
}

MaskEdit::~MaskEdit()
{
    delete ui;
}


void MaskEdit::updateFrame(cv::Mat *frame, std::vector<cv::Point> *points){
    capture = frame;
    rectPoints = points;
    if(frame != nullptr && points != nullptr){
        ratioX = (float)capture->size().width / ui->lblFrame->width();
        ratioY = (float)capture->size().height / ui->lblFrame->height();
    }
    update();
}

void MaskEdit::updateFrame(cv::Mat *frame){
    updateFrame(frame, rectPoints);
}

void MaskEdit::updateFrame(std::vector<cv::Point> *points){
    updateFrame(capture, points);
}


void MaskEdit::paintEvent(QPaintEvent *){

    if(capture != nullptr){

        QImage img = QImage((uchar *)capture->data, (int)capture->cols, (int)capture->rows, static_cast<int>(capture->step.buf[0]), QImage::Format_RGB888);
        QPixmap px = QPixmap::fromImage(img);
        QPainter maskPainter(&px);

        // QPen: style(), width(), brush(), capStyle() and joinStyle().
        QPen pen(Qt::green, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        maskPainter.setPen(pen);

        QPolygon poly;
        int size = 10;

        for(uint i=0; i<rectPoints->size(); i++){

            int x = rectPoints->at(i).x;
            int y = rectPoints->at(i).y;
            poly << QPoint(x, y);
            maskPainter.drawEllipse(x-(size/2), y-(size/2), size, size);
        }

        pen.setStyle(Qt::DashLine);
        maskPainter.setPen(pen);
        maskPainter.drawPolygon(poly);
        maskPainter.end();

        ui->lblFrame->setPixmap(px);
    }
}


bool MaskEdit::eventFilter(QObject *object, QEvent *ev)
{
    if (ev->type() == QEvent::MouseButtonPress){

        selectedCornerIndex = selectCorner((QMouseEvent*)ev);
    }
    else if (ev->type() == QEvent::MouseButtonRelease){

        selectedCornerIndex = -1;
    }
    else if (ev->type() == QEvent::MouseMove){

        QMouseEvent* eMouse = (QMouseEvent*)ev;
        if (eMouse->type() == QMouseEvent::MouseMove){

            UpdateSelectedCornerPosition(eMouse);
        }
    }

    (void)object; // var unused
    return false;
}



int MaskEdit::selectCorner(QMouseEvent* eMouse){

    if(capture != nullptr && rectPoints != nullptr){

        int margin = 10;

        // x and y relative to the captured frame, not the UI
        int mouseX = (int)( (float)eMouse->x() * ratioX );
        int mouseY = (int)( (float)eMouse->y() * ratioY );
        int marginX = (int)( margin * ratioX );
        int marginY = (int)( margin * ratioY );

        for(uint i=0; i<rectPoints->size(); i++){

            int x = rectPoints->at(i).x;
            int y = rectPoints->at(i).y;

            if( (x-marginX <= mouseX && mouseX <= x+marginX) &&
                (y-marginY <= mouseY && mouseY <= y+marginY) )
                {

                return (int)i;
            }
        }
    }

    return -1;
}


void MaskEdit::UpdateSelectedCornerPosition(QMouseEvent* eMouse){

    if(capture != nullptr && rectPoints != nullptr){
        if(selectedCornerIndex != -1){
            if( (0 <= eMouse->x() && eMouse->x() < ui->lblFrame->width()) &&
                (0 <= eMouse->y() && eMouse->y() < ui->lblFrame->height()) )
                {

                int mouseX = (int)( (float)eMouse->x() * ratioX );
                int mouseY = (int)( (float)eMouse->y() * ratioY );
                rectPoints->at(selectedCornerIndex).x = mouseX;
                rectPoints->at(selectedCornerIndex).y = mouseY;
                update();
            }
        }
    }
}

