#include "monitorgui.h"
#include "ui_monitorgui.h"

MonitorGui::MonitorGui(SandboxSetup *_setup, QWidget *parent) :
    SubApp("Monitors", "Cancel resolution", parent),
    ui(new Ui::MonitorGui)
{
    setup = _setup;
    ui->setupUi(this);
    resolution = new QRect;

    std::string path = ".monitors.tmp";
    monitors = std::map<std::string, std::vector<std::string>>();
    initMonitorMapWithFile(path);
    QList<QScreen*> screens = QApplication::screens();

    for(int i=0; i<screens.size(); i++){
        QScreen* sc = screens[i];
        ui->cbxOutputs->addItem( sc->name() );
    }

    if(screens.size() > 0){
        loadResolutionsOf(screens[0]);
        showMonitorsScreenshot(screens[0]);
    }
    ui->txtInstructions->setAlignment(Qt::AlignJustify);
}

MonitorGui::~MonitorGui()
{
    delete resolution;
    delete ui;
}

void MonitorGui::valideRoutine(){

    QString selectedRes = ui->cbxResolutions->currentText();
    outputName = ui->cbxOutputs->currentText();
    std::vector<std::string> res = splitResolution(selectedRes.toStdString());
    width = std::stoi(res[0]);
    height = std::stoi(res[1]);

    QScreen *sc = getMonitor();
    resolution->setX(sc->geometry().x());
    resolution->setY(sc->geometry().y());
    resolution->setWidth(width);
    resolution->setHeight(height);

    endSuccess = true;
    setup->getBeamer()->setResolution(cv::Size(getWidth(), getHeight()));
}

void MonitorGui::cancelRoutine(){

    endSuccess = false;
}

QScreen* MonitorGui::getMonitor(){

    QScreen *sc = QApplication::screens().at(0);
    QList<QScreen *> lst = sc->virtualSiblings();


    for(int i=0; i<lst.size(); i++){
      std::string name = lst[i]->name().toStdString();
      if(name == outputName.toStdString()){
          sc = lst.at(i);
          break;
      }
    }

    return sc;
}

void MonitorGui::on_cbxOutputs_currentIndexChanged(int index)
{
    QList<QScreen*> screens = QApplication::screens();
    QScreen *sc = screens[index];
    loadResolutionsOf(sc);
    showMonitorsScreenshot(sc);
}


void MonitorGui::initMonitorMapWithFile(std::string path){

    // Save in file the monitors and their resolutions
    int err = system( ("xrandr -d :0 | sed -n '1!p' > " + path).c_str() );
    if(err){
        std::cout << "Couldn't get screens and their resolutions" << std::endl;
        exit(err);
    }
    loadResolutionsFromFile(path);
    std::remove(path.c_str());
}

// load resolutions into GUI
void MonitorGui::loadResolutionsOf(QScreen* sc){
    ui->cbxResolutions->clear();
    const char *key = sc->name().toStdString().c_str();
    std::vector<std::string> resolutions = monitors[key];

    for(uint i=0; i<resolutions.size(); i++){
        std::vector<std::string> res = splitResolution(resolutions[i]);
        ui->cbxResolutions->addItem( QString::fromStdString(res[0] + "x" + res[1]) );
    }
}

// Get height and width from the string "_width_x_height_"
// http://www.martinbroadhurst.com/how-to-split-a-string-in-c.html
std::vector<std::string> MonitorGui::splitResolution(std::string s){
    char delim = 'x';
    std::vector<std::string> res;
    std::stringstream ss(s);
    std::string token;
    while (std::getline(ss, token, delim)) {
        res.push_back(token);
    }

    return res;
}

bool MonitorGui::isResolution(std::string s){
    return splitResolution(s).size() == 2;
}

void MonitorGui::loadResolutionsFromFile(std::string path){
    // Get monitors and their resolutions into a Map
    std::ifstream infile(path);
    std::string line;
    std::string key;

    while (std::getline(infile, line)){

        std::istringstream iss(line);
        std::string l;
        iss >> l;

        if(!isResolution(l)){
            key = l;
            monitors[key] = std::vector<std::string>();
        }else{
            monitors[key].push_back(l);
        }
    }
}

void MonitorGui::showMonitorsScreenshot(QScreen *screen){
    QPixmap px = screen->grabWindow(0);
    ui->lblScreenshot->setPixmap(px);
}
