#ifndef SANDBOX_PROJECTION_H
#define SANDBOX_PROJECTION_H

#include <opencv2/opencv.hpp>
#include "beamer.h"
#include "camera.h"

class Projection{
    private:
        cv::Mat adjustingMatrix;
        float distanceTopSandbox;

        void deprojectPixelsFromDepth(cv::Mat_<float> &depth, cv::Rect mask, Camera *camera, cv::Point3f beamer_pos, cv::Mat_<cv::Point2i> &deprojectMap);
        void filterLowestDeprojectedPoints(cv::Mat_<float> &depth, cv::Mat_<cv::Point2i> &deprojectMap, cv::Mat_<cv::Point2i> &frameMapMask);
        void buildFrame(cv::Mat_<float> &depth, cv::Mat_<cv::Point2i> &frameMapMask, cv::Mat_<cv::Vec3b> &src, cv::Mat_<cv::Vec3b> &dst);
        void buildFrame2(cv::Mat_<cv::Point2i> &frameMapMask, cv::Mat_<cv::Vec3b> &src, cv::Mat_<cv::Vec3b> &dst);
        cv::Point2i findMatchingPixel(int i, int j, float z, Camera *camera, cv::Point3f beamer_pos);
        void copyPixelsInto(cv::Point2i pixel_dst, cv::Mat_<cv::Vec3b> &dst, cv::Point2i pixel_src, cv::Mat_<cv::Vec3b> &src, cv::Mat_<float> &depth);
        cv::Size getMatchingSize(cv::Mat &src, cv::Mat &base);


    public:
        Projection();

        void setAdjustingMatrix(cv::Mat matrix){ adjustingMatrix = matrix; }
        cv::Mat getAdjustingMatrix(){ return adjustingMatrix; }
        void setDistanceTopSandbox(float dist){ distanceTopSandbox = dist; };
        float getDistanceTopSandbox(){ return distanceTopSandbox; };

        cv::Point2i rotatePixel(cv::Point2i pixel);
        cv::Point2i revertRotatePixel(cv::Point2i pixel);
        void adjustFrame(cv::Mat_<float> depth, cv::Mat_<cv::Vec3b> src, cv::Mat_<cv::Vec3b> &dst, Camera *camera, cv::Point3f beamer_pos);
        void adjustFrame2(cv::Mat_<float> depth, cv::Mat_<cv::Vec3b> src, cv::Mat_<cv::Vec3b> &dst, Camera *camera, cv::Point3f beamer_pos);
        void printAdjustingMatrix();

};
#endif