#ifndef SANDBOX_CONFIG_H
#define SANDBOX_CONFIG_H

#include <yaml-cpp/yaml.h>
#include <opencv2/opencv.hpp>
#include "camera.h"
#include "projection.h"
#include "beamer.h"

#define SCFG_MASK "CroppingMask"
#define SCFG_POSITION "BeamerPosition"
#define SCFG_MATRIX "AdjustingMatrix"
#define SCFG_DISTANCE "DistanceTopSandbox"
#define SCFG_RESOLUTION "BeamerResolution"
#define SCFG_PROCESS_PROFIL "FrameProcessProfil"

class SandboxConfig{
    private:
        SandboxConfig(){};

    public:
        static int saveAdjustingMatrixInto(char *path, cv::Mat matrix);
        static int saveDistanceToSandboxTop(char *path, float distance);
        static int saveCroppingMaskInto(char *path, cv::Rect mask);
        static int saveBeamerPositionInto(char *path, cv::Point3f position);
        static int saveBeamerResolutionInto(char *path, cv::Size resolution);
        static int saveFrameProcessProfil(char *path, FrameProcessProfil profil);

        static int loadAdjustingMatrixFrom(char *path, Projection *projection);
        static int loadDistanceToSandboxTop(char *path, Projection *projection);
        static int loadCroppingMaskFrom(char *path, Camera *camera);
        static int loadBeamerPositionFrom(char *path, Beamer *beamer);
        static int loadBeamerResolutionFrom(char *path, Beamer *beamer);
        static int loadFrameProcessProfil(char *path, FrameProcessProfil *profil);

};

#endif
