#ifndef SANDBOX_SETUP_H
#define SANDBOX_SETUP_H

#include <opencv2/opencv.hpp>
#include "beamer.h"
#include "projection.h"
#include "camera.h"
#include "sandboxConfig.h"

class SandboxSetup{
    private:
        char *defaultConfigFilePath = (char *)"./sandbox_conf.yaml";
        Projection *projection;
        Camera *camera;
        Beamer *beamer;

        double toDegrees(double radians);

    public:
        SandboxSetup();
        ~SandboxSetup();

        Projection* getProjection(){ return projection; };
        Camera* getCamera(){ return camera; };
        Beamer* getBeamer(){ return beamer; };
        
        
        // save config in file => persistant
        int saveConfigInto(char *path);
        int saveConfig();
        
        int loadFrameProcessProfilFrom(char *path);
        int loadFrameProcessProfil();
        int loadCroppingMask(char *path);
        int loadCroppingMask();

        // edit variables of config => not persistant
        void setupAdjustMatrix(std::vector<cv::Point> rectPoints, cv::Point center);
        void setupCroppingMask(std::vector<cv::Point> rectPoints);
};

#endif
