#include "../../inc/sandboxSetup.h"


SandboxSetup::SandboxSetup(){
    projection = new Projection();
    camera = new Camera();
    beamer = new Beamer();
}

SandboxSetup::~SandboxSetup(){
    delete beamer;
    delete camera;
    delete projection;
}

//
//  PUBLIC
//


// return 1 when config can't be saved in file
int SandboxSetup::saveConfigInto(char *path){
    if(SandboxConfig::saveAdjustingMatrixInto(path, projection->getAdjustingMatrix()))
        return 1;
    
    if(SandboxConfig::saveDistanceToSandboxTop(path, projection->getDistanceTopSandbox()))
        return 1;

    if(SandboxConfig::saveCroppingMaskInto(path, camera->getCroppingMask()))
        return 1;

    if(SandboxConfig::saveBeamerResolutionInto(path, cv::Size(beamer->getWidth(), beamer->getHeight())))
        return 1;

    if(SandboxConfig::saveBeamerPositionInto(path, beamer->getPosition()))
        return 1;

    if(SandboxConfig::saveFrameProcessProfil(path, *beamer->getProfil()))
        return 1;

    return 0;
}


int SandboxSetup::saveConfig(){
    return saveConfigInto(defaultConfigFilePath);
}

int SandboxSetup::loadCroppingMask(char *path){
    return SandboxConfig::loadCroppingMaskFrom(path, camera);
}

int SandboxSetup::loadCroppingMask(){
    return loadCroppingMask(defaultConfigFilePath);
}

int SandboxSetup::loadFrameProcessProfilFrom(char *path){
    return SandboxConfig::loadFrameProcessProfil(path, beamer->getProfil());
}

int SandboxSetup::loadFrameProcessProfil(){
    return loadFrameProcessProfilFrom(defaultConfigFilePath);
}

/*  Assuming points positions are :
 *      pts[0] : top left
 *      pts[1] : bottom left
 *      pts[2] : bottom right
 *      pts[3] : top right
 * 
 *  center : center of the rotation in the projected frame
 */
void SandboxSetup::setupAdjustMatrix(std::vector<cv::Point> rectPoints, cv::Point center){

    // Set adjusting matrix for the projection
    int widthTop = rectPoints[3].x - rectPoints[0].x;
    double angle1 = atan((double)(rectPoints[3].y - rectPoints[0].y) / widthTop);
    cv::Mat matRotation = cv::getRotationMatrix2D(center, toDegrees(angle1), 1); // adjustingMatrix
    projection->setAdjustingMatrix(matRotation);
}

/*  Assuming points positions are :
 *      pts[0] : top left
 *      pts[1] : bottom left
 *      pts[2] : bottom right
 *      pts[3] : top right
 */
void SandboxSetup::setupCroppingMask(std::vector<cv::Point2i> rectPoints){

    // Set cropping mask
    int widthTop = rectPoints[3].x - rectPoints[0].x;
    cv::Point2i pbl = projection->rotatePixel(rectPoints[1]);
    cv::Point2i pbr = projection->rotatePixel(rectPoints[2]);
    cv::Point2i pbot = (pbl.y < pbr.y) ? pbl : pbr;
    cv::Point2i ptop = projection->rotatePixel(rectPoints[0]);

    cv::Size rectSize = cv::Size(widthTop, pbot.y - rectPoints[0].y);
    camera->setCroppingMask(cv::Rect(ptop, rectSize)); // croppingMask
}



//
//  PRIVATE
//


double SandboxSetup::toDegrees(double radians){
    return radians * (180.0 / M_PI);
}
